
#include "cminimap.h"

#include <QGraphicsView>
#include <QGraphicsRectItem>
#include <QGraphicsPixmapItem>
#include <QGridLayout>
#include <QRectF>

#include <QDebug>


const int minW = 200;
const int minH = 200;


CMiniMap::CMiniMap(QWidget *parent) :
    QWidget(parent),
    GraphicsView(new QGraphicsView),
    GraphicsScene(new QGraphicsScene),
    ViewPortRect(NULL),
    CurrentPixmap(NULL),
    ViewPortColor(Qt::white)
{
    GraphicsView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    GraphicsView->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    GraphicsView->setFrameShape(QFrame::NoFrame);
    GraphicsView->setMinimumSize(minW, minH);
    GraphicsView->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
    GraphicsView->setScene(GraphicsScene);

    QGridLayout *layout = new QGridLayout(this);
    layout->setSpacing(0);
    layout->setMargin(0);
    layout->addWidget(GraphicsView);
}



void CMiniMap::mousePressEvent(QMouseEvent *event)
{
    if (event->button() == Qt::LeftButton) {

        qreal offsetX = qreal(width() - GraphicsView->minimumWidth()) / 2.0;
        qreal offsetY = qreal(height() - GraphicsView->minimumHeight()) / 2.0;

        //Передаём относительные координаты курсора
        emit viewPortPositionChanged(QPointF(
                                         (event->posF().x() - offsetX) / qreal(GraphicsView->minimumWidth()),
                                         (event->posF().y() - offsetY) / qreal(GraphicsView->minimumHeight())));
    }
}



void CMiniMap::setViewPortColor(const QColor &color)
{
    ViewPortColor = color;

    if (ViewPortRect)
        ViewPortRect->setPen(ViewPortColor);
}


void CMiniMap::updateImage(const QImage &image)
{
    if (CurrentPixmap && CurrentPixmap->pixmap().size() == image.size())
        CurrentPixmap->setPixmap(QPixmap::fromImage(image));
    else
        setImage(image);
}


void CMiniMap::setImage(const QImage &image)
{

    GraphicsScene->clear();
    GraphicsView->resetTransform();

    ViewPortRect = NULL;
    CurrentPixmap = NULL;

    if (!image.isNull()) {

        const int w = image.width();
        const int h = image.height();

        GraphicsScene->setSceneRect(0, 0, w, h);
        CurrentPixmap = GraphicsScene->addPixmap(QPixmap::fromImage(image));
        ViewPortRect = GraphicsScene->addRect(0, 0, w, h, QPen(QBrush(ViewPortColor), 10));

        const int border = 10;

        qreal sx = (GraphicsView->width() - border) / qreal(w);
        qreal sy = (GraphicsView->height() - border) / qreal(h);
        qreal scale = qMin(sx, sy);
        //qDebug() << scale;

        GraphicsView->scale(scale, scale);
        GraphicsView->setMinimumSize(w*scale, h*scale);

    }

}


void CMiniMap::setViewPortRect(const QRectF &rect)
{
    //qDebug() << Q_FUNC_INFO << sender();
    if (ViewPortRect) {
        ViewPortRect->setRect(rect);
        //qDebug() << "Rect" << rect;
    }
}

